require "brains/snapdragonbrain"
require "stategraphs/SGSnapdragon"

local assets=
{
    Asset("ANIM", "anim/snapdragon_build_frozen.zip"),
    Asset("ANIM", "anim/snapdragon_build_neck.zip"),
    Asset("ANIM", "anim/snapdragon_build.zip"),
    Asset("SOUND", "sound/beefalo.fsb"),
}

local prefabs =
{
}

SetSharedLootTable( 'snapdragon',
{
    {'whisperpod', .6},
    {'plantmeat',        1.00},
})
SetSharedLootTable( 'snapdragon_buddy',
{
    {'whisperpod', 1},
    {'plantmeat',        1},
    {'plantmeat',        1},
})

local periodictable = 
{
    {'dragonfruit_seeds',      0.1},
    {'corn_seeds',            1},
    {'pumpkin_seeds',            1},
    {'watermelon_seeds',            1},
    {'eggplant_seeds',            1},
    {'durian_seeds',            1},
    {'pomegranate_seeds',            1},
    {'sweet_potato_seeds', 1},
    {"radish_seeds", 1}
}
local sounds = 
{
    walk = "dontstarve/creatures/eyeplant/eye_emerge",
    grunt = "UCSounds/snapdragon/grunt",
    yell = "UCSounds/snapdragon/yell",
    swish = "dontstarve/beefalo/tail_swish",
    curious = "dontstarve/beefalo/curious",
    angry = "UCSounds/snapdragon/angry",
}


local function OnFreeze(inst)
    inst.AnimState:SetBuild("snapdragon_build_frozen")
end


local function OnThaw(inst)
	if inst.seeds ~= nil then
		inst:DoTaskInTime(0, function(inst) inst.AnimState:OverrideSymbol("hair", "snapdragon_build_"..inst.seeds, "hair") end)
		inst:DoTaskInTime(0, function(inst) inst.AnimState:OverrideSymbol("ear", "snapdragon_build_"..inst.seeds, "ear") end)
		inst:DoTaskInTime(0, function(inst) inst.AnimState:OverrideSymbol("face", "snapdragon_build_"..inst.seeds, "face") end)
		inst:DoTaskInTime(0, function(inst) inst.AnimState:OverrideSymbol("jaw", "snapdragon_build_"..inst.seeds, "jaw") end)
	end
end

local function OnUnFreeze(inst)
	if inst.seeds ~= nil then
		inst:DoTaskInTime(0, function(inst) inst.AnimState:SetBuild("snapdragon_build_"..inst.seeds) end)
		inst:DoTaskInTime(0, function(inst) inst.AnimState:OverrideSymbol("hair", "snapdragon_build_"..inst.seeds, "hair") end)
		inst:DoTaskInTime(0, function(inst) inst.AnimState:OverrideSymbol("ear", "snapdragon_build_"..inst.seeds, "ear") end)
		inst:DoTaskInTime(0, function(inst) inst.AnimState:OverrideSymbol("face", "snapdragon_build_"..inst.seeds, "face") end)
		inst:DoTaskInTime(0, function(inst) inst.AnimState:OverrideSymbol("jaw", "snapdragon_build_"..inst.seeds, "jaw") end)
	end
end


local function RetargetFn(inst)

    if not inst.isHostile then
        return nil
    end
    local IsBuddy = inst:HasTag("snapdragon_buddy")
    local SEE_TARGET_DIST = 5
    if IsBuddy then
        SEE_TARGET_DIST = 20
    end

    return FindEntity(inst, SEE_TARGET_DIST,
        function(guy)

                local basicReqs = (
                not guy:HasTag("snapdragon") and not guy:HasTag("snapdragon_buddy") and
                not guy:HasTag("bramble") and
                guy.components.health and 
                not guy.components.health:IsDead() and 
                inst.components.combat:CanTarget(guy) and 
                not (inst.components.follower and inst.components.follower.leader ~= nil and guy:HasTag("abigail"))) and
                not (inst.components.follower and guy.components.follower and inst.components.follower.leader ~= nil and inst.components.follower.leader == guy.components.follower.leader) and
                not (inst.components.follower and guy.components.follower and inst.components.follower.leader ~= nil and guy.components.follower.leader and guy.components.follower.leader.components.inventoryitem and guy.components.follower.leader.components.inventoryitem.owner and inst.components.follower.leader == guy.components.follower.leader.components.inventoryitem.owner)

                if IsBuddy and basicReqs and (guy:HasTag("monster") or guy:HasTag("spider_monkey") or guy:HasTag("hostile") or guy:HasTag("flytrap")) then
                    return true
                end
                if not IsBuddy and basicReqs and guy:HasTag("character") and not (guy:HasTag("monster") or guy:HasTag("spider_monkey")) then
                    return true
                end

                return false      	        
        end)
end

local function CheckLushSeason(inst, data)
    if inst:HasTag("snapdragon_buddy") then
        return
    end
    if data.season == SEASONS.LUSH then
        inst.isHostile = true
        inst.AnimState:OverrideSymbol("neck", "snapdragon_build_neck", "neck")
    else
        inst.isHostile = false
        inst.AnimState:OverrideSymbol("neck", "snapdragon_build", "neck")
    end
end

local function OnNewTarget(inst, data)
    if data and data.target and (data.target:HasTag("snapdragon") or data.target:HasTag("snapdragon_buddy")) then
        inst.components.combat:SetTarget(nil)
    end
    if inst.components.follower and data and data.target and data.target == inst.components.follower.leader then
        inst.components.follower:SetLeader(nil)
    end
end

local function CanShareTarget(dude)
    return dude:HasTag("snapdragon") and not dude.components.health:IsDead()
end

local function OnAttacked(inst, data)
    if data.attacker ~= nil then
        inst.components.combat:SetTarget(data.attacker)
        inst.components.combat:ShareTarget(data.attacker, 30, CanShareTarget, 5)
    end
end

local function OnEat(inst, data)
    -- Increase the amount of food in the stomach.
    inst.foodItemsEatenCount = inst.foodItemsEatenCount + 1
end

local function GetStatus(inst)
    if inst.components.follower.leader ~= nil then
        return "FOLLOWER"
    end
end

local function ShouldAcceptItem(inst, item)
    if inst.components.sleeper:IsAsleep() then
        return false
    end

    if inst.components.combat.target then
        return false
    end

    if item.components.edible then
        if (item.components.edible.foodtype == "VEGGIE")
           and inst.components.follower.leader
           and inst.components.follower:GetLoyaltyPercent() > 0.9 then
            return false
        end
    end

    if item.components.edible and item.components.edible.foodtype == 'VEGGIE' and item.prefab =="radish" then
        return true
    end
    return false
end

local function OnGetItemFromPlayer(inst, giver, item)
    if item.components.edible then
        if item.components.edible.foodtype == "VEGGIE" then
            if giver.components.leader then
                inst.sg:GoToState("eat")
				inst.SoundEmitter:PlaySound("dontstarve/common/makeFriend")
				giver.components.leader:AddFollower(inst)
                inst.components.follower:AddLoyaltyTime(item.components.edible:GetHunger() * TUNING.PIG_LOYALTY_PER_HUNGER)
            end
        end
    end
end

local function OnRefuseItem(inst, item)
   
end

local function fn(Sim)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	local sound = inst.entity:AddSoundEmitter()
	inst.sounds = sounds
    
	local shadow = inst.entity:AddDynamicShadow()
	shadow:SetSize( 6, 2 )
    inst.Transform:SetFourFaced()
    inst.foodItemsEatenCount = 0
    inst.isHostile = false
    
    MakePoisonableCharacter(inst)
    MakeCharacterPhysics(inst, 100, .5)
    local scale = 1.22
    inst.Transform:SetScale(scale, scale, scale)
    
    inst:AddTag("snapdragon")

    anim:SetBank("snapdragon")
    anim:SetBuild("snapdragon_build")
    anim:PlayAnimation("idle", true)

    inst:AddTag("animal")
    inst:AddTag("largecreature")
    
    inst:AddComponent("eater")
    inst.components.eater:SetVegetarian()
    inst.components.eater:SetOnEatFn(OnEat)
    
    inst:AddComponent("combat")
    inst.components.combat.hiteffectsymbol = "beefalo_body"
    inst.components.combat:SetDefaultDamage(TUNING.BEEFALO_DAMAGE.DEFAULT)
    inst.components.combat:SetRetargetFunction(1, RetargetFn)
     
    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(TUNING.BEEFALO_HEALTH)

    inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetChanceLootTable('snapdragon')    
    
    inst:AddComponent("inspectable")
    inst.components.inspectable.getstatus = GetStatus
    
    inst:AddComponent("knownlocations")
    inst:AddComponent("herdmember")
    inst.components.herdmember.herdprefab = "snapdragonherd"
    
    inst:AddComponent("leader")
    inst:AddComponent("follower")
    inst.components.follower.maxfollowtime = TUNING.BEEFALO_FOLLOW_TIME
    inst:ListenForEvent("newcombattarget", OnNewTarget)
    inst:ListenForEvent("attacked", OnAttacked)

    inst:AddComponent("periodicspawner")
    inst.components.periodicspawner.chancetable = periodictable
    inst.components.periodicspawner:SetRandomTimes(200, 300)
    inst.components.periodicspawner:SetDensityInRange(20, 2)
    inst.components.periodicspawner:SetMinimumSpacing(8)
    inst.components.periodicspawner:Start()

    inst:AddComponent("trader")
    inst.components.trader:SetAcceptTest(ShouldAcceptItem)
    inst.components.trader.onaccept = OnGetItemFromPlayer
    inst.components.trader.onrefuse = OnRefuseItem
    inst.components.trader.deleteitemonaccept = true

    inst:AddComponent("locomotor") -- locomotor must be constructed before the stategraph
    inst.components.locomotor.walkspeed = 4
    

    inst:AddComponent("sleeper")
    inst.components.sleeper:SetResistance(3)
    
    local brain = require "brains/snapdragonbrain"
    inst:SetBrain(brain)
    inst:SetStateGraph("SGSnapdragon")

    MakeLargeBurnableCharacter(inst, "swap_fire")
    MakeLargeFreezableCharacter(inst, "beefalo_body")

    inst:ListenForEvent("freeze", OnFreeze)
    inst:ListenForEvent("seasonChange", function(it, data) CheckLushSeason(inst, data) end, GetWorld())

    return inst
end

local function Rename(inst)
    inst.components.named:SetName(STRINGS.NAMES["SNAPDRAGON_BUDDY_"..inst.seeds]) 
end

local function buddy_fn()
    local inst = fn(1)
	
	if inst.seeds == nil then
		inst.seeds = "pale"
	end
	
    inst:AddComponent("named")
    inst:AddTag("snapdragon_buddy")
	inst:DoTaskInTime(0, Rename)

    inst:RemoveComponent("leader")
    inst:RemoveComponent("follower")
    inst:RemoveComponent("trader")
    inst:RemoveTag("snapdragon")

    inst:AddComponent("follower")

	--inst.components.lootdropper:AddChanceLoot((inst.seeds.."_vomit"), 1)
	
    inst.components.locomotor.walkspeed = 2.77
    inst.components.locomotor:SetTriggersCreep(false)
    inst.isHostile = true
    inst.components.lootdropper:SetChanceLootTable('snapdragon_buddy')

    return inst
end

return Prefab( "forest/animals/snapdragon", fn, assets, prefabs),
Prefab("snapdragon_buddy", buddy_fn, assets, prefabs) 
