require "prefabutil"

local assets =
{
    Asset("ANIM", "anim/whisperpod.zip"),
    Asset("ATLAS", "images/inventoryimages/whisperpod.xml"),	
    Asset("IMAGE", "images/inventoryimages/whisperpod.tex"),
}

local prefabs =
{
    "plant_normal_ground",
}

local notags = {'NOBLOCK', 'player', 'FX'}
local function test_ground(inst, pt)
	local ground_OK = inst:GetIsOnLand(pt.x, pt.y, pt.z)
	local tiletype = GetGroundTypeAtPosition(pt)
    
    ground_OK = ground_OK and tiletype ~= GROUND.INTERIOR and tiletype ~= GROUND.GASJUNGLE and 
                      tiletype ~= GROUND.COBBLEROAD and tiletype ~= GROUND.FOUNDATION and 
                      tiletype ~= GROUND.ROCKY and tiletype ~= GROUND.ROAD and tiletype ~= GROUND.IMPASSABLE and
						tiletype ~= GROUND.UNDERROCK and tiletype ~= GROUND.WOODFLOOR and 
						tiletype ~= GROUND.CARPET and tiletype ~= GROUND.CHECKER and tiletype < GROUND.UNDERGROUND
	
	if ground_OK then
	    local ents = TheSim:FindEntities(pt.x,pt.y,pt.z, 4, nil, notags) -- or we could include a flag to the search?
		local min_spacing = inst.components.deployable.min_spacing or 2

	    for k, v in pairs(ents) do
			if v ~= inst and v:IsValid() and v.entity:IsVisible() and not v.components.placer and v.parent == nil then
				if distsq( Vector3(v.Transform:GetWorldPosition()), pt) < min_spacing*min_spacing then
					return false
				end
			end
		end
		return true
	end
	return false
end

local function OnDeploy(inst, pt)
    local plant = SpawnPrefab("whisperpod_normal_ground")
    plant.Transform:SetPosition(pt.x, 0, pt.z)
    plant.SoundEmitter:PlaySound("dontstarve/wilson/plant_seeds")
    inst:Remove()
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()

    MakeInventoryPhysics(inst)

    inst.AnimState:SetBank("whisperpod")
    inst.AnimState:SetBuild("whisperpod")
    inst.AnimState:PlayAnimation("idle")
    inst.AnimState:SetRayTestOnBB(true)

    MakeInventoryFloatable(inst)
	
    inst:AddTag("deployedplant")
	inst:AddTag("plant")

    inst:AddComponent("deployable")
    inst.components.deployable.test = test_ground
    inst.components.deployable.ondeploy = OnDeploy
	
    inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/inventoryimages/whisperpod.xml"

    --inst.name = "whisperpod"

    inst:AddComponent("inspectable")
	
    inst:AddComponent("fuel")
    inst.components.fuel.fuelvalue = TUNING.MED_FUEL

    MakeSmallBurnable(inst, TUNING.SMALL_BURNTIME)
    MakeSmallPropagator(inst)

    return inst
end

return Prefab("whisperpod", fn, assets, prefabs),
    MakePlacer("whisperpod_placer", "whisperpod_normal_ground", "whisperpod_normal_ground", "placer")
