require "prefabutil"

local assets =
{
	Asset("ANIM", "anim/whisperpod_normal_ground.zip"),
}

local prefabs =
{
    "snapdragon",
    "snapdragonherd",
}
local function onmatured(inst)

    inst.SoundEmitter:PlaySound("dontstarve/common/farm_harvestable")
	
	local pos = inst:GetPosition()
    local snappy = SpawnPrefab("snapdragon_buddy")
	snappy.Transform:SetPosition(pos:Get())
	snappy.sg:GoToState("create")
	
	local snapplant = SpawnPrefab("snapplant")
	snapplant.Transform:SetPosition(pos:Get())
	snapplant.Bind(snapplant, snappy)
	
	snappy:DoTaskInTime(0, function(snappy) snappy.AnimState:OverrideSymbol("hair", "snapdragon_build_"..inst.planted or "snapdragon_build", "hair") end)
	snappy:DoTaskInTime(0, function(snappy) snappy.AnimState:OverrideSymbol("ear", "snapdragon_build_"..inst.planted or "snapdragon_build", "ear") end)
	snappy:DoTaskInTime(0, function(snappy) snappy.AnimState:OverrideSymbol("face", "snapdragon_build_"..inst.planted or "snapdragon_build", "face") end)
	snappy:DoTaskInTime(0, function(snappy) snappy.AnimState:OverrideSymbol("jaw", "snapdragon_build_"..inst.planted or "snapdragon_build", "jaw") end)
	snappy.seeds = inst.planted or "pale"
	
	inst.components.workable:SetWorkAction(nil)
	inst:Remove()
end

local function onburnt(inst)
    DefaultBurntFn(inst)
end

local function OnDigUp(inst)
    local x, y, z = inst.Transform:GetWorldPosition()
    local product = SpawnPrefab("whisperpod")
	product.Transform:SetPosition(x, y, z)
	
    inst:Remove()
end

local function GetStatus(inst)
    return inst.growing and "GROWING" or "GENERIC"
end

local function Colors(color)
	local colors = {
		"orange",
		"yellow",
		"pink",
		"red",
		"green",
		"purple",
		"pale"
	}
	return colors[math.random(1, #colors)]
end
	
--------------------------------------------------------------------------

local function OnGetItemFromPlayer(inst, giver, item)
    if item.components.plantable ~= nil and not inst.growing then
		inst:AddComponent("crop")
		inst.components.crop:StartGrowing("snapdragon", .5) --TUNING.RABBIT_RESPAWN_TIME
		inst.components.crop:SetOnMatureFn(onmatured)
		inst.growing = true
		
		inst.planted = Colors(item.prefab)		
				
		if inst.planted ~= nil then
			inst.AnimState:SetBuild("snapdragon_build_"..inst.planted)
		else
			inst.planted = "pale"
			inst.AnimState:SetBuild("snapdragon_build_"..inst.planted)
		end
    end
end

local function AcceptTest(inst, item, giver)
    return item.components.plantable ~= nil and not inst.growing
end

local function OnSave(inst, data)
    data.planted = inst.planted
	data.growing = inst.growing
end

local function OnPreLoad(inst, data)
    if data ~= nil and data.growing then
        inst.growing = data.growing
    end
	
    if data ~= nil and data.planted then
        inst.planted = data.planted
    end
end

local function OnInit(inst, data)
    if not inst.growing then
		inst:RemoveComponent("crop")
    end
	
	
    if inst.planted ~= nil then
		inst.AnimState:SetBuild("snapdragon_build_"..inst.planted)
	end
end

--------------------------------------------------------------------------

local function fn()
	local inst = CreateEntity()

	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()

	inst.entity:AddMiniMapEntity()
	inst.MiniMapEntity:SetIcon("plant_normal_ground.png")
			
	inst.AnimState:SetBank("whisperpod_normal_ground")
	inst.AnimState:SetBuild("whisperpod_normal_ground")
	inst.AnimState:PlayAnimation("sprout")

	inst:AddTag("whisperpod")
	inst:AddTag("NPC_workable")
	inst:AddTag("trader")
	inst:AddTag("plant")
	
	inst:AddComponent("trader")
	inst.components.trader:SetAcceptTest(AcceptTest)
	inst.components.trader.onaccept = OnGetItemFromPlayer
	
	inst:AddComponent("crop")
	inst.components.crop:SetOnMatureFn(onmatured)
		
	inst:AddComponent("inspectable")
	inst.components.inspectable.getstatus = GetStatus
	--inst.components.inspectable.nameoverride = "plant_normal"

	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.DIG)
	inst.components.workable:SetWorkLeft(1)
	inst.components.workable:SetOnFinishCallback(OnDigUp)

	MakeSmallBurnable(inst)
	MakeSmallPropagator(inst)
	inst.components.burnable:SetOnBurntFn(onburnt)
	inst.components.burnable:SetOnIgniteFn(nil)
	inst.components.burnable:SetOnExtinguishFn(nil)
	
	inst.growing = false
	inst.planted = nil
	--inst.name = "Snapdragon Plant"
	
    inst.OnSave = OnSave
    inst.OnPreLoad = OnPreLoad
	
	inst:DoTaskInTime(0, OnInit)

	return inst
end
	
return Prefab("whisperpod_normal_ground", fn, assets, prefabs)